/* ESP8266_OpenWeatherMap
 *  
 * Időjárási adatok lekérése az OpenWeatherMap szerverről
 * (ingynes regisztrációval beszerezheztó API kulcs kell hozzá.
 * A kapott adatokat megszűrjük, csak a main szekció
 * temp, pressure és humidity adatait tarjuk meg és íratjuk ki.
 * 
 * Tested with Arduino IDE 1.8.19 and ESP32 Arduino core 2.0.2
 */
#include <WiFi.h>
#include <HTTPClient.h>
#include <ArduinoJson.h>
#include "secrets.h"

StaticJsonDocument<500> doc;
StaticJsonDocument<200> filter;
String server_url = "http://api.openweathermap.org/data/2.5/weather?q=Debrecen&mode=json&APPID=" + OPENWEATHERMAP_APPID;

void setup () {
  Serial.begin(115200);
  WiFi.mode(WIFI_STA);
  WiFi.begin(WIFI_SSID, WIFI_PASS);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
  }

  // Prepare filter
  filter["main"]["temp"] = true;
  filter["main"]["pressure"] = true;
  filter["main"]["humidity"] = true;
}

void loop() {
  if (WiFi.status() == WL_CONNECTED) {     // Check WiFi connection status
    HTTPClient http;                       // Declare an object of class HTTPClient
    http.begin(server_url);                // Specify request destination
    int httpCode = http.GET();             // Send the request
    if (httpCode > 0) {                    // Check the returning code
      String payload = http.getString();   // Get the request response payload
      Serial.println(payload);             // Print the response payload
      DeserializationError error = deserializeJson(doc, payload, DeserializationOption::Filter(filter));
      // Test if parsing succeeds.
      if (error) {
        Serial.print(F("deserializeJson() failed: "));
        Serial.println(error.f_str());
      } else {
        float tempK = doc["main"]["temp"];
        int hPa = doc["main"]["pressure"];
        int relHum = doc["main"]["humidity"];
        Serial.println(F("deserializeJson() succeeded! "));
        Serial.println("Filtered list:");
        serializeJsonPretty(doc, Serial);
        Serial.print("\r\ntemperature [°C] = ");
        Serial.println(tempK - 273.16, 1);
        Serial.print("pressure [hPa] = ");
        Serial.println(hPa);
        Serial.print("rel. humidity [%] = ");
        Serial.println(relHum);
      }
    }
    http.end();    //Close connection
  }
  delay(10000);    //Send a request every 10 seconds
}
