/* ESP32_ThingSpeak_JSON 
 *  
 * Egyszerű ThingSpeak kliens, amely egy DHT22 
 * szenzorral mért adatait (hőmérséklet, rel. páratartalom)
 * POST  metódussal, JSON formátumban küldi el a szerverre.
 * A DHT22 szenzor a GPIO23 kivezetéshez csatlakozik.
 * 
 * Tested with Arduino IDE 1.8.19 and ESP32 Arduino Core 2.0.2
 * /

#include <WiFi.h>
#include <HTTPClient.h>
#include <ArduinoJson.h>
#include "DHT.h"               // Adafruit DHT library
#include "secrets.h"           // Wifi jelszó, ThingSpeak Write API key

#define DHTPIN          23     // GPIO23 pin a DHT szenzorhoz
DHT dht(DHTPIN, DHT22);

StaticJsonDocument<500> doc;
const char* serverName = "https://api.thingspeak.com/update.json"; // formátumfüggő cím!

void setup() {
  Serial.begin(115200);        // Soros port inicialálás
  dht.begin();                 // DHT szenzor indítása
  WiFi.mode(WIFI_STA);         // Station (client) mód
  WiFi.begin(WIFI_SSID, WIFI_PASS);
  Serial.print("Connecting to ");
  Serial.print(WIFI_SSID);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println();
  Serial.print("Connected! IP address: ");
  Serial.println(WiFi.localIP());
  doc["api_key"] = THINGSPEAK_WRITE_APIKEY;
  doc["field1"] = 0.0;         // Hőmérséklet Celsius fokokban
  doc["field2"] = 0.0;         // Relatív páratartalom [%]
}

void loop() {
  String httpRequestData;
  float tnew = dht.readTemperature();         // Hőmérséklet kiolvasása
  if (!isnan(tnew)) doc["field1"]  = tnew;
  float hnew = dht.readHumidity();            // Páratartalom kiolvasása
  if (!isnan(hnew)) doc["field2"]  = hnew;
  serializeJson(doc, httpRequestData);
  serializeJsonPretty(doc, Serial);
  if (WiFi.status() == WL_CONNECTED) {        // Ha a WiFi hálózat elérhető
    HTTPClient http;                          // Webkliens példányosítása
    http.begin(serverName);                   // Szerver név megadása
    http.addHeader("Content-Type", "application/json");
    int httpResponseCode = http.POST(httpRequestData); // adatok kiküldése
    Serial.print("HTTP Response code: ");     // Válaszkód kiírása
    Serial.println(httpResponseCode);         // Sikeres küldés esetén 200-as kód
    http.end();                               // Erőforrások felszabadítása
  }
  else {
    Serial.println("WiFi Disconnected");
  }
  delay(20000);
}
